package it.unitn.disi.prog2.bonizzi;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToggleGroup;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.stage.Stage;
import javafx.util.Duration;

/**
 * 
 * Classe che estende {@link import javafx.stage.Stage}. È la finestra
 * principale del programma, ovvero dove verranno visualizzati i tasti di comando
 * e la griglia contenete i cerchi
 * 
 * @author Alberto Bonizzi
 */
public class MainStage extends Stage {
    
    private final int ROWS;
    private Circle[][] matrix;
    
    private GridPane grid;
    
    private Timeline autoStep;
    
    /**
     * 
     * Costruttore della classe
     * 
     * @param rows un intero che segnala il numero di righe e colonne che
     * dovranno comparire nella tabella
     */
    
    public MainStage (int rows) {
        
        ROWS = rows;
        matrix = new Circle[ROWS][ROWS];
        
        BorderPane root;
        HBox box1, box2;
        Scene scene;
        
        ToggleButton alg1, alg2, alg3;
        final ToggleGroup group;
        final Button print, clear, step, start, stop;
        
        root = new BorderPane ();
        
        box1 = new HBox ();
        box1.setPadding (new Insets (10, 0, 10, 0));
        box1.setSpacing (5);
        box1.setAlignment (Pos.CENTER);
        
        group = new ToggleGroup ();
        
        alg1 = new ToggleButton ("Alg 1");
        alg1.setSelected (true);
        alg1.setToggleGroup (group);
        alg1.setUserData (new Alg1Handler ());
        alg1.setMinWidth (80);
        
        alg2 = new ToggleButton ("Alg 2");
        alg2.setToggleGroup (group);
        alg2.setUserData (new Alg2Handler ());
        alg2.setMinWidth (80);
        
        alg3 = new ToggleButton ("Alg 3");
        alg3.setToggleGroup (group);
        alg3.setUserData (new Alg3Handler ());
        alg3.setMinWidth (80);
        
        box1.getChildren ().addAll (alg1, alg2, alg3);
        
        root.setTop (box1);
        
        grid = new GridPane ();
        grid.setHgap (2);
        grid.setVgap (2);
        grid.setAlignment (Pos.CENTER);
        grid.setPadding (new Insets (10));
        
        grid.setGridLinesVisible (true);
        
        for (int i=0; i<ROWS; i++) {
            for (int j=0; j<ROWS; j++) {
                
                Circle c;
                
                c = new Circle (20);
                c.setOpacity (0);
                c.setStroke (Color.gray (0.3));
                c.setStrokeWidth (2);
                //c.setFill (Color.gray (0.6));
                c.setFill (Color.LIGHTBLUE);
                
                grid.add (c, j, i);
                matrix[i][j] = c;
               
                
                c.setOnMouseClicked (new EventHandler<MouseEvent> () {

                    @Override
                    public void handle (MouseEvent event) {
                        
                        
                        Circle c;
                        
                        
                        c = (Circle) event.getSource ();
                        c.setOpacity (1-c.getOpacity ());
                    }
                });
            }
        }
        
        root.setCenter (grid);
        
        box2 = new HBox ();
        box2.setPadding (new Insets (10));
        box2.setSpacing (5);
        box2.setAlignment (Pos.CENTER);
        
        print = new Button ("Print");
        print.setMinWidth (80);
        
        clear = new Button ("Clear");
        clear.setMinWidth (80);
        
        step = new Button ("Step");
        step.setMinWidth (80);
        
        start = new Button ("Start");
        start.setMinWidth (80);
        
        stop = new Button ("Stop");
        stop.setMinWidth (80);
        
        
        print.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                new PrintStage (matrix).show ();
            }
        });
        
        clear.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                clearGrid ();
            }
        });
        
        step.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                MovementHandler handler;
                ToggleButton b;
                
                b = (ToggleButton) group.getSelectedToggle ();
                
                
                if (b != null) {
                        
                    handler = (MovementHandler) b.getUserData ();
                    handler.moveBalls (matrix);
                    
                }
            }
        
        });
        
        autoStep = new Timeline (new KeyFrame (Duration.seconds (1), new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                step.fireEvent (new ActionEvent ());
            }
            
        }));
        
        autoStep.setCycleCount (Timeline.INDEFINITE);
        
        start.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                autoStep.play ();
            }
            
            
        });
        
        stop.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                
                autoStep.pause ();
            }
            
            
        });
        
        box2.getChildren ().addAll (print, clear, step, start, stop);
        
        root.setBottom (box2);
        
        scene = new Scene (root);
        
        scene.addEventHandler (KeyEvent.KEY_PRESSED, new EventHandler<KeyEvent> () {

            @Override
            public void handle (KeyEvent event) {
                
                KeyCode code;
                
                code = event.getCode ();
                
                if (code == KeyCode.C) {
                    
                    clear.fireEvent (new ActionEvent ());
                
                } else if (code == KeyCode.S) {
                    
                    step.fireEvent (new ActionEvent ());
                }
            }



        });
        
        setTitle ("Esame di Bonizzi Alberto - 164491");
        setScene (scene);
        
        
    }
    
    /**
     * 
     * Funzione che setta l'opacità di ogni cerchio nella tabella a 0 
     * tramite {@link javafx.scene.shape.Circle#setOpacity(double) } in modo
     * da pulire la tabella
     * 
     */
    
    private void clearGrid () {
        
        for (int i=0; i<ROWS; i++) {
            for (int j=0; j<ROWS; j++) {
                
                matrix[i][j].setOpacity (0);
            }
        }
    }
}
