package it.unitn.disi.prog2.bonizzi;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;
import javafx.stage.Stage;

/**
 *
 * Classe principale del progetto che deve essere lanciata. Mostra un popup
 * nel quale è possibile specificare la dimensione della tabella.
 * Successivamente, lancia una istanza di {@link it.unitn.disi.prog2.bonizzi.MainStage}
 * 
 * @author Alberto Bonizzi
 */
public class MyApp extends Application {
    
    @Override
    public void start (final Stage primaryStage) {

        VBox root;
        Text prompt;
        final TextField textField;
        final Button button;

        root = new VBox ();
        root.setPadding (new Insets (5));
        root.setSpacing (5);
        root.setAlignment (Pos.CENTER);
        root.setMaxWidth (300);

        prompt = new Text ("Scegli la dimensione:");
        textField = new TextField ();
        textField.setAlignment (Pos.CENTER);
        button = new Button ("Conferma");

        root.getChildren ().addAll (prompt, textField, button);

        primaryStage.setScene (new Scene (root));
        primaryStage.setTitle ("Scegli la dimensione");
        primaryStage.show ();

        button.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {

                int rows;
                
                try {
                    
                    rows = Integer.parseInt (textField.getText ());
                    
                    primaryStage.hide ();
                    new MainStage (rows).show ();
                    
                } catch (NumberFormatException e) {
                    
                    popupError ("Numero inserito non valido!");
                }
                
            }

        });
        
        textField.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                button.fireEvent (new ActionEvent ());
            }
            
            
        });
        
    }
    
    /**
     * 
     * Funzione semplice che crea un popup di errore con un bottone
     * "Ok" che quando premuto chiude il popup
     * 
     * @param message il messaggio di errore da visualizzare bel popup
     */
    
    private void popupError (String message) {
        
        final Stage stage;
        VBox pane;
        Button close;
        
        stage = new Stage ();
        pane = new VBox ();
        pane.setPadding (new Insets (10));
        pane.setSpacing (20);
        pane.setAlignment (Pos.CENTER);
        
        close = new Button ("Ok");
        close.setMaxWidth (50);
        
        close.setOnAction (new EventHandler<ActionEvent> () {

            @Override
            public void handle (ActionEvent event) {
                stage.hide ();
            }
            
        });
        
        pane.getChildren ().addAll (new Text (message), close);
        
        stage.setTitle ("Errore");
        stage.setScene (new Scene (pane));
        stage.show ();
    }

    /**
     * The main() method is ignored in correctly deployed JavaFX application.
     * main() serves only as fallback in case the application can not be
     * launched through deployment artifacts, e.g., in IDEs with limited FX
     * support. NetBeans ignores main().
     *
     * @param args the command line arguments
     */
    public static void main (String[] args) {
        launch (args);
    }
    
}
